/*
 * main.c - main hdw-sniff
 *
 * Copyright (C) 2004/05 hackbard@hackdaworld.org
 *
 */

#include "main.h"

// void parse_package(unsigned char *ptr,const struct pcap_pkthdr *pcap_header,const unsigned char *package);

/* functions */

int display_console(t_info *info,char *string) {

  int x,y;
  t_display *display;

  display=(t_display *)&(info->display);

  x=display->max_x-1;
  y=display->max_y-1;

  display_line(display,0,0,2,0,'-');
  display_string(display,4,0,"sta",3);
  display_line(display,8,0,x,0,'-');

  //display_line(display,0,0,0,y,'|');
  //display_line(display,x,0,x,y,'|');

  display_line(display,0,y-1,x,y-1,'-');

  display_draw(display);

  return 23;
}

int noop(t_input *input,void *ptr) {

  return 23;
}

int get_user_interaction(t_info *info) {

  char *string;

  string=(char *)malloc(info->display.max_x*sizeof(char));

  input_get_event(&(info->input),noop,info);

  /*
  if(info->input.content[0]=='h') {
    display_console(info,"hdw-sniff help:");
    display_console(info,"h - print this help");
    display_console(info,"x - enable/disable hex output");
    display_console(info,"a - enable/disable ascii output");
    display_console(info,"q - quit");
    display_console(info,"console navigation: arrow up/down");
    display_console(info,"network list navigation: page up/down");
  }
  */

  if(info->input.content[0]=='x') {
    info->mode^=MODE_HEXOUT;
    display_console(info,"toggled hex output");
  }

  else if(info->input.content[0]=='a') {
    info->mode^=MODE_ASCIIOUT;
    display_console(info,"toggled ascii output");
  }

  else if(info->input.content[0]=='q') {
    event_stop(&(info->event));
    display_console(info,"quit!");
  }

  else {
    snprintf(string,info->display.max_x,"unknown event (%x)",
             info->input.content[0]);
    display_console(info,string);
  }

  return 23;
}

int react_on_event(t_event *event,void *ptr) {

  t_info *info;

  info=(t_info *)ptr;

  if(event_check(event,0)==E_FD_YES) get_user_interaction(info);
  else pcap_dispatch(info->pcap_handle,-1,parse_package,(u_char *)ptr);

  return 23;
}

int usage(void) {
  puts("usage: hdw-sniff <options>");
  puts("\toptions:");
  puts("\t\t-m <mode> \tmonitor and/or wlanng");
  puts("\t\t-d <device> \twlan0,eth0");
  puts("\t\t-l <logfile>");
  puts("\t\t-k <key> \t(string)");
  puts("\t\t-D <file> \t(dump packages to file)");
  puts("\t\t-h \tdisplay this help message");
  puts("");

  return 23;
}

int hop_channel(t_event *event,void *ptr) {
 
  struct iwreq iwreq;
  t_info *info;

  info=(t_info *)ptr;

  if(info->current_channel>=CHANNEL_MAX) info->current_channel=1;
  memset(&iwreq,0,sizeof(iwreq));
  strcpy(iwreq.ifr_name,info->device);
  iwreq.u.freq.e=0;
  iwreq.u.freq.m=info->current_channel;
  if(ioctl(info->channel_hop_fd,SIOCSIWFREQ,&iwreq)<0) {
    puts("unable to hop channel");
    perror("ioctl");
    return -23;
  }
 ++(info->current_channel);
 return 23;
}

int main(int argc, char **argv) {

  t_info info;
  int pcap_fd;
  int i;
  char sys_call[MAX_SYSCALL_CHARS];
  char pcap_error[PCAP_ERRBUF_SIZE];

  memset(&info,0,sizeof(t_info));
 
 /* parse arguments */
 for(i=1;i<argc;i++) {
  if(argv[i][0]=='-') {
   switch(argv[i][1]) {
    case 'h':
     usage();
    case 'm':
     if(!strncmp(argv[i+1],"monitor",7)) {
      info.mode|=MODE_MONITOR;
      puts("will go to monitor mode.");
     }
     else if(!strncmp(argv[i+1],"wlanng",6)) {
      info.mode|=MODE_WLANNG;
      puts("expecting wlanng header in package.");
     }
     else {
      printf("unknown mode: %s\n",argv[1]);
      return -23;
     }
     ++i;
     break;
    case 'l':
     if((info.log_fd=open(argv[i+1],O_RDWR|O_CREAT))!=0)
      printf("logfile -> %s\n",argv[i+1]);
     else
      puts("warning: can't write to logfile.");
     ++i;
     break;
    case 'D':
     if((info.dump_fd=open(argv[i+1],O_RDWR|O_CREAT))!=0)
      printf("dump file -> %s\n",argv[i+1]);
     else
      puts("warning: can't dump to file.");
     ++i;
     break;
    case 'd':
     strncpy(info.device,argv[i+1],MAX_DEV_CHARS);
     ++i;
     break;
    default:
     usage();
     return -23;
   }
  } else {
   usage();
   return -23;
  }
 }

 /* setting up device */
 if(info.mode&MODE_MONITOR) {
  sprintf(sys_call,"iwconfig %s mode monitor",info.device);
  puts("set monitoring mode ...");
  system(sys_call);
 }
 sprintf(sys_call,"ifconfig %s up",info.device);
 puts("setting up device ...");
 system(sys_call);

 if(info.log_fd==0) {
 if((info.log_fd=open("/tmp/hdw-sniff.log",O_RDWR|O_CREAT))!=0)
   puts("using logfile /tmp/hdw-sniff.log ...");
 else {
   puts("failed to open logfile ...");
   return -23;
 }

 /* pcap */
 if((info.pcap_handle=pcap_open_live(info.device,BUFSIZ,1,-1,pcap_error))==NULL)
 {
  printf("%s: %s\n",argv[0],pcap_error);
  return -23;
 }
 pcap_fd=pcap_fileno(info.pcap_handle);
 /* -> non blocking? */

 /* socket fd for channel hopping */
 info.channel_hop_fd=socket(AF_INET,SOCK_DGRAM,0);

 display_init(&(info.display),info.log_fd);

 input_init(&(info.input),info.log_fd);
 //input.mode=CONTENT_BUFFER;
 input_ios_init(&info.input);

 event_init(&(info.event),info.log_fd);
 event_set_timeout(&(info.event),HOP_SEC,HOP_USEC);

 event_math(0,&(info.event),READ,ADD);
 event_math(pcap_fd,&(info.event),READ,ADD);

 list_init(&(info.sniffed_sta),info.log_fd);

 display_console(&info,"foo");

 event_start(&(info.event),&info,react_on_event,hop_channel);

 input_shutdown(&(info.input));
 display_shutdown(&(info.display));
 
 puts("");
 puts("");
 puts("thanks for using hdw-sniff (C) 2004/05 hackbard");
 puts("");
 puts("bugreports: hackbard@hackdaworld.org");

 return 23;

}
